import Big, { RoundingMode } from 'big.js';
import { SetOptional } from 'type-fest';

interface AssetConstructorParams {
  key: string;
  originSymbol: string;
}
declare class Asset {
  readonly key: string;
  readonly originSymbol: string;
  constructor({ key, originSymbol }: AssetConstructorParams);
  isEqual(asset: Asset): boolean;
}

interface AssetAmountParams {
  amount: bigint;
  decimals: number;
  symbol?: string;
}
interface AssetAmountConstructorParams
  extends AssetConstructorParams,
    AssetAmountParams {}
declare class AssetAmount extends Asset {
  readonly amount: bigint;
  readonly decimals: number;
  readonly symbol: string;
  constructor({
    amount,
    decimals,
    symbol,
    ...other
  }: AssetAmountConstructorParams);
  static fromAsset(asset: Asset, params: AssetAmountParams): AssetAmount;
  isSame(asset: AssetAmount): boolean;
  isEqual(asset: AssetAmount): boolean;
  copyWith(params: Partial<AssetAmountConstructorParams>): AssetAmount;
  toBig(): Big;
  toBigDecimal(maxDecimal?: number, roundType?: RoundingMode): Big;
  toDecimal(maxDecimal?: number, roundType?: RoundingMode): string;
}

type ChainAssetId =
  | string
  | number
  | bigint
  | {
      [key: string]: ChainAssetId;
    };
interface ChainAssetsData {
  asset: Asset;
  balanceId?: ChainAssetId;
  decimals?: number;
  id?: ChainAssetId;
  metadataId?: ChainAssetId;
  minId?: ChainAssetId;
  palletInstance?: number;
  min?: number;
}

interface ParachainConstructorParams
  extends SetOptional<ChainConstructorParams, 'type'> {
  assetsData?: Map<string, ChainAssetsData> | ChainAssetsData[];
  genesisHash: string;
  parachainId: number;
  ss58Format: number;
  weight?: number;
  ws: string;
}
declare class Parachain extends Chain {
  readonly assetsData: Map<string, ChainAssetsData>;
  readonly genesisHash: string;
  readonly parachainId: number;
  readonly ss58Format: number;
  readonly weight: number;
  readonly ws: string;
  constructor({
    assetsData,
    genesisHash,
    parachainId,
    ss58Format,
    weight,
    ws,
    type,
    ...others
  }: ParachainConstructorParams);
  getAssetId(asset: Asset): ChainAssetId;
  getBalanceAssetId(asset: Asset): ChainAssetId;
  getMinAssetId(asset: Asset): ChainAssetId;
  getMetadataAssetId(asset: Asset): ChainAssetId;
  getAssetPalletInstance(asset: Asset): number | undefined;
  getAssetDecimals(asset: Asset): number | undefined;
  getAssetMin(asset: Asset): number;
}

interface EvmParachainConstructorParams
  extends Omit<ParachainConstructorParams, 'type'> {
  id: number;
  rpc: string;
}
declare class EvmParachain extends Parachain {
  readonly id: number;
  readonly rpc: string;
  constructor({ id, rpc, ...others }: EvmParachainConstructorParams);
}

type AnyChain = Parachain | EvmParachain;
type AnyParachain = Parachain | EvmParachain;
declare enum ChainType {
  'Parachain' = 'parachain',
  'EvmParachain' = 'evm-parachain',
}
declare enum Ecosystem {
  Polkadot = 'polkadot',
  Kusama = 'kusama',
  AlphanetRelay = 'alphanet-relay',
}

interface ChainConstructorParams {
  ecosystem?: Ecosystem;
  isTestChain?: boolean;
  key: string;
  name: string;
  type: ChainType;
}
declare abstract class Chain {
  readonly ecosystem?: Ecosystem;
  readonly isTestChain: boolean;
  readonly key: string;
  readonly name: string;
  readonly type: ChainType;
  constructor({
    ecosystem,
    isTestChain,
    key,
    name,
    type,
  }: ChainConstructorParams);
  isParachain(): this is Parachain;
  isEvmParachain(): this is EvmParachain;
}

export {
  AnyChain,
  AnyParachain,
  Asset,
  AssetAmount,
  AssetAmountConstructorParams,
  AssetAmountParams,
  AssetConstructorParams,
  Chain,
  ChainAssetId,
  ChainAssetsData,
  ChainConstructorParams,
  ChainType,
  Ecosystem,
  EvmParachain,
  EvmParachainConstructorParams,
  Parachain,
  ParachainConstructorParams,
};
